;; Generated package description from vertico.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "vertico" "2.14.0.20260907.4" "VERTical Interactive COmpletion" '((emacs "29.1") (compat "31")) :commit "8581ed12e9190005ea9afaef19f2a22951aa1bfb" :authors '(("Daniel Mendler" . "mail@daniel-mendler.de")) :maintainer '("Daniel Mendler" . "mail@daniel-mendler.de") :keywords '("convenience" "files" "matching" "completion") :url "https://github.com/minad/vertico")
